# 12 "java/java_lexer.mll"
 

  open Java_parser
  open Lexing
  open Java_ast

  type location = position * position

  let loc lexbuf : location = 
    (lexeme_start_p lexbuf, lexeme_end_p lexbuf)

  exception Lexical_error of location * string
  exception Syntax_error of location
  exception Dotdot of string


  let lex_error lexbuf s = raise (Lexical_error(loc lexbuf,s))

  let buf = Buffer.create 97

  let kw_table = 
    let table = Hashtbl.create 17 in
    let _ = 
      List.iter
	(fun (s,t) -> Hashtbl.add table s t)
	[ "abstract", ABSTRACT;
	  "assert", ASSERT;
	  "assigns", ASSIGNS;
	  "axiom", AXIOM;
	  "behavior", BEHAVIOR;
	  "boolean", BOOLEAN;
	  "break", BREAK;
	  "byte", BYTE;
	  "byvalue", BYVALUE;
	  "case", CASE;
	  "cast", CAST;
	  "catch", CATCH;
	  "char", CHAR;
	  "class", CLASS;
	  "const", CONST;
	  "continue", CONTINUE;
	  "decreases", DECREASES;
	  "default", DEFAULT;
	  "do", DO;
	  "double", DOUBLE;
	  "else", ELSE;
	  "ensures", ENSURES;
	  "extends", EXTENDS;
	  "false", FALSE;
	  "final", FINAL;
	  "finally", FINALLY;
	  "float", FLOAT;
	  "for", FOR;
	  "future", FUTURE;
	  "generic", GENERIC;
	  "ghost", GHOST;
	  "goto", GOTO;
	  "if", IF;
	  "implements", IMPLEMENTS;
	  "import", IMPORT;
	  "inner", INNER;
	  "instanceof", INSTANCEOF;
	  "int", INT;
	  "integer", INTEGER;
	  "interface", INTERFACE;
	  "invariant", INVARIANT;
	  "logic", LOGIC;
	  "long", LONG;
	  "loop_invariant", LOOP_INVARIANT;
	  "native", NATIVE;
	  "new", NEW;
	  "null", NULL;
	  "operator", OPERATOR;
	  "outer", OUTER;
	  "package", PACKAGE;
	  "predicate", PREDICATE;
	  "private", PRIVATE;
	  "protected", PROTECTED;
	  "public", PUBLIC;
	  "reads", READS;
	  "real", REAL;
	  "requires", REQUIRES;
	  "rest", REST;
	  "return", RETURN;
	  "short", SHORT;
	  "signals", SIGNALS;
	  "static", STATIC;
	  "super", SUPER;
	  "switch", SWITCH;
	  "synchronized", SYNCHRONIZED;
	  "this", THIS;
	  (* "threadsafe" ? *)
	  "throw", THROW;
	  "throws", THROWS;
	  "transient", TRANSIENT;
	  "true", TRUE;
	  "try", TRY;
	  "var", VAR;
	  "void", VOID;
	  "volatile", VOLATILE;
	  "while", WHILE;	
	]
    in table

  let id_or_kw s =
    try
      let k = Hashtbl.find kw_table s in
      (*i
	prerr_string "Keyword ";
	prerr_endline s;
	i*)
      k
    with
	Not_found ->
	  (*i
	    prerr_string "Ident ";
	    prerr_endline s;
	    i*)
	  (ID s)

  let special_kw_table = 
    let table = Hashtbl.create 17 in
    let _ = 
      List.iter
	(fun (s,t) -> Hashtbl.add table s t)
	[ (* "everything", BSEVERYTHING ; *)
	  "exists", BSEXISTS ;
	  (* "fresh", BSFRESH ; *)
	  "forall", BSFORALL ;
	  "nothing", BSNOTHING;
	  (*
	  "fields_of", BSFIELDSOF;
          "not_conditionally_updated", BSNOTCONDITIONALLYUPDATED;
	  *)
	  "old", BSOLD;
	  "result", BSRESULT;
	  (*"type", BSTYPE;
	  "typeof", BSTYPEOF;
	  "fpi", BSFPI; *)
	]
    in table

  let special_id lexbuf s =
    try
      Hashtbl.find special_kw_table s
    with
	Not_found ->
	  lex_error lexbuf ("unknown special keyword \\"^s)

(*

  let jml_spec_token base jml_string =
(*i
    Format.fprintf Config.log "In file %s, parsing JML Spec: %s@."
      (Location.base_filename base) jml_string;
i*)
    match Jml_syntax.parse_jml_specification base jml_string with 
    | Ast_types.Jml_declaration d -> JML_DECLARATIONS(d)
    | Ast_types.Jml_method_specification s -> JML_METHOD_SPECIFICATION(s)
    | Ast_types.Jml_loop_annotation la -> JML_LOOP_ANNOTATION(la)
    | Ast_types.Jml_assertion a -> JML_ASSERTION(a)
    | Ast_types.Jml_annotation_statement Ast_types.Set_statement s -> JML_ANNOTATION_STATEMENT(Ast_types.Set_statement(s))
    | Ast_types.Jml_axiom(id,e) -> JML_AXIOM(id,e)
    | Ast_types.Jml_type(id) -> JML_TYPE(id)
    | Ast_types.Jml_logic_reads(id,t,p,r) -> JML_LOGIC_READS(id,t,p,r)
    | Ast_types.Jml_logic_def(id,t,p,e) -> JML_LOGIC_DEF(id,t,p,e)
    assert false

*)
  let newline lexbuf =
    let pos = lexbuf.lex_curr_p in
    lexbuf.lex_curr_p <- 
      { pos with pos_lnum = pos.pos_lnum + 1; 
	  pos_bol = pos.pos_cnum }


# 179 "java/java_lexer.ml"
let __ocaml_lex_tables = {
  Lexing.lex_base = 
   "\000\000\188\255\189\255\190\255\191\255\192\255\193\255\194\255\
    \195\255\196\255\004\000\081\000\110\000\078\000\227\255\228\255\
    \003\000\230\255\067\000\081\000\082\000\083\000\107\000\085\000\
    \086\000\108\000\109\000\171\000\246\255\247\255\196\000\254\000\
    \129\000\254\255\003\000\221\255\138\000\085\000\253\255\250\255\
    \001\000\141\000\252\255\077\001\152\001\201\255\186\000\162\001\
    \185\001\219\255\242\255\218\255\241\255\222\255\220\255\214\255\
    \235\255\212\255\234\255\092\000\233\255\094\000\111\000\111\000\
    \232\255\217\255\206\255\213\255\208\255\113\000\216\255\116\000\
    \215\255\218\001\228\001\253\001\027\002\054\002\205\255\204\255\
    \083\002\203\255\202\255\115\002\144\002\238\001\154\002\007\002\
    \200\255\139\000\179\002\182\002\141\000\142\000\198\255\249\002\
    \016\003\039\003\144\000\197\255\199\255\055\002\018\003\019\003\
    \040\003\255\255\245\000\251\255\248\000\137\000\178\002\179\002\
    \058\002\138\000\246\002";
  Lexing.lex_backtrk = 
   "\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\066\000\050\000\050\000\044\000\255\255\255\255\
    \026\000\255\255\024\000\045\000\032\000\019\000\018\000\017\000\
    \015\000\012\000\011\000\010\000\255\255\255\255\007\000\066\000\
    \016\000\255\255\000\000\255\255\255\255\004\000\255\255\255\255\
    \255\255\255\255\255\255\006\000\054\000\255\255\255\255\054\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\048\000\255\255\029\000\046\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\030\000\255\255\031\000\
    \255\255\255\255\053\000\255\255\255\255\051\000\255\255\255\255\
    \052\000\255\255\255\255\255\255\053\000\255\255\055\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\056\000\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\004\000\003\000\
    \255\255\255\255\255\255\255\255\002\000\003\000\255\255\003\000\
    \001\000\004\000\255\255";
  Lexing.lex_default = 
   "\002\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\089\000\255\255\255\255\255\255\000\000\000\000\
    \255\255\000\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\000\000\000\000\255\255\255\255\
    \255\255\000\000\255\255\000\000\040\000\255\255\000\000\000\000\
    \040\000\041\000\000\000\255\255\255\255\000\000\255\255\255\255\
    \255\255\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\255\255\000\000\255\255\255\255\255\255\
    \000\000\000\000\000\000\000\000\000\000\255\255\000\000\255\255\
    \000\000\255\255\255\255\255\255\255\255\255\255\000\000\000\000\
    \255\255\000\000\000\000\255\255\255\255\255\255\255\255\255\255\
    \000\000\255\255\255\255\255\255\255\255\255\255\000\000\255\255\
    \255\255\255\255\255\255\000\000\000\000\102\000\102\000\102\000\
    \038\000\000\000\108\000\000\000\108\000\255\255\111\000\111\000\
    \255\255\255\255\255\255";
  Lexing.lex_trans = 
   "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\034\000\033\000\039\000\034\000\034\000\000\000\000\000\
    \034\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \034\000\018\000\003\000\034\000\000\000\023\000\022\000\010\000\
    \009\000\008\000\024\000\026\000\028\000\025\000\027\000\032\000\
    \012\000\011\000\011\000\011\000\011\000\011\000\011\000\011\000\
    \011\000\011\000\014\000\029\000\019\000\020\000\013\000\015\000\
    \067\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\005\000\031\000\004\000\016\000\030\000\
    \090\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\007\000\021\000\006\000\017\000\074\000\
    \066\000\011\000\011\000\011\000\011\000\011\000\011\000\011\000\
    \011\000\011\000\011\000\068\000\069\000\061\000\062\000\059\000\
    \057\000\056\000\054\000\053\000\039\000\038\000\073\000\042\000\
    \050\000\052\000\060\000\065\000\074\000\078\000\077\000\077\000\
    \077\000\077\000\077\000\077\000\077\000\077\000\075\000\075\000\
    \055\000\051\000\049\000\037\000\063\000\064\000\070\000\071\000\
    \036\000\072\000\100\000\073\000\094\000\094\000\073\000\099\000\
    \105\000\105\000\078\000\000\000\000\000\078\000\035\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\076\000\000\000\
    \000\000\000\000\041\000\000\000\000\000\000\000\000\000\058\000\
    \000\000\000\000\000\000\073\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\078\000\044\000\044\000\044\000\044\000\044\000\
    \044\000\044\000\044\000\044\000\044\000\048\000\076\000\048\000\
    \000\000\000\000\047\000\047\000\047\000\047\000\047\000\047\000\
    \047\000\047\000\047\000\047\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\000\000\033\000\
    \001\000\255\255\255\255\000\000\255\255\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\109\000\
    \000\000\000\000\255\255\030\000\000\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\000\000\000\000\000\000\000\000\043\000\000\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\000\000\000\000\000\000\000\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\000\000\
    \000\000\000\000\255\255\000\000\000\000\255\255\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \000\000\000\000\000\000\000\000\043\000\000\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \044\000\044\000\044\000\044\000\044\000\044\000\044\000\044\000\
    \044\000\044\000\047\000\047\000\047\000\047\000\047\000\047\000\
    \047\000\047\000\047\000\047\000\045\000\046\000\045\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\045\000\000\000\
    \045\000\047\000\047\000\047\000\047\000\047\000\047\000\047\000\
    \047\000\047\000\047\000\000\000\000\000\107\000\000\000\000\000\
    \255\255\000\000\000\000\000\000\045\000\046\000\045\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\087\000\045\000\087\000\
    \045\000\000\000\086\000\086\000\086\000\086\000\086\000\086\000\
    \086\000\086\000\086\000\086\000\074\000\074\000\074\000\074\000\
    \074\000\074\000\074\000\074\000\074\000\074\000\084\000\084\000\
    \084\000\084\000\084\000\084\000\084\000\084\000\084\000\084\000\
    \082\000\083\000\082\000\074\000\000\000\075\000\075\000\075\000\
    \075\000\075\000\075\000\075\000\075\000\075\000\075\000\086\000\
    \086\000\086\000\086\000\086\000\086\000\086\000\086\000\086\000\
    \086\000\042\000\073\000\114\000\103\000\000\000\000\000\114\000\
    \082\000\083\000\082\000\080\000\080\000\080\000\080\000\080\000\
    \080\000\080\000\080\000\080\000\080\000\000\000\000\000\000\000\
    \000\000\105\000\114\000\000\000\080\000\080\000\080\000\080\000\
    \080\000\080\000\073\000\000\000\074\000\000\000\077\000\077\000\
    \077\000\077\000\077\000\077\000\077\000\077\000\075\000\075\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\038\000\073\000\080\000\080\000\080\000\080\000\
    \080\000\080\000\079\000\080\000\080\000\080\000\080\000\080\000\
    \080\000\080\000\080\000\080\000\080\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\104\000\080\000\080\000\080\000\080\000\
    \080\000\080\000\000\000\073\000\000\000\000\000\085\000\081\000\
    \085\000\000\000\079\000\084\000\084\000\084\000\084\000\084\000\
    \084\000\084\000\084\000\084\000\084\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\080\000\080\000\080\000\080\000\
    \080\000\080\000\000\000\000\000\112\000\255\255\000\000\081\000\
    \084\000\084\000\084\000\084\000\084\000\084\000\084\000\084\000\
    \084\000\084\000\086\000\086\000\086\000\086\000\086\000\086\000\
    \086\000\086\000\086\000\086\000\082\000\092\000\082\000\000\000\
    \000\000\000\000\093\000\000\000\113\000\255\255\088\000\000\000\
    \088\000\107\000\255\255\000\000\000\000\000\000\095\000\095\000\
    \095\000\095\000\095\000\095\000\095\000\095\000\095\000\095\000\
    \000\000\000\000\107\000\255\255\082\000\000\000\082\000\095\000\
    \095\000\095\000\095\000\095\000\095\000\000\000\088\000\114\000\
    \088\000\000\000\000\000\114\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\092\000\
    \000\000\000\000\000\000\000\000\000\000\092\000\114\000\000\000\
    \000\000\092\000\000\000\000\000\255\255\255\255\000\000\000\000\
    \000\000\092\000\000\000\000\000\000\000\092\000\000\000\092\000\
    \091\000\096\000\096\000\096\000\096\000\096\000\096\000\096\000\
    \096\000\096\000\096\000\000\000\255\255\255\255\038\000\039\000\
    \000\000\000\000\096\000\096\000\096\000\096\000\096\000\096\000\
    \097\000\097\000\097\000\097\000\097\000\097\000\097\000\097\000\
    \097\000\097\000\033\000\000\000\000\000\000\000\000\000\033\000\
    \000\000\097\000\097\000\097\000\097\000\097\000\097\000\098\000\
    \098\000\098\000\098\000\098\000\098\000\098\000\098\000\098\000\
    \098\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \098\000\098\000\098\000\098\000\098\000\098\000\255\255\255\255\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\033\000\000\000\000\000\000\000\
    \000\000\000\000\033\000\000\000\000\000\000\000\033\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\033\000\000\000\
    \000\000\000\000\033\000\000\000\033\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\039\000\255\255\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\255\255\255\255\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \255\255";
  Lexing.lex_check = 
   "\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\000\000\000\000\040\000\034\000\000\000\255\255\255\255\
    \034\000\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \000\000\000\000\000\000\034\000\255\255\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \016\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \010\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\011\000\
    \018\000\011\000\011\000\011\000\011\000\011\000\011\000\011\000\
    \011\000\011\000\011\000\013\000\013\000\019\000\019\000\020\000\
    \021\000\022\000\023\000\024\000\036\000\037\000\011\000\041\000\
    \026\000\025\000\059\000\061\000\012\000\011\000\012\000\012\000\
    \012\000\012\000\012\000\012\000\012\000\012\000\012\000\012\000\
    \022\000\025\000\026\000\032\000\062\000\063\000\069\000\069\000\
    \032\000\071\000\089\000\012\000\092\000\093\000\011\000\098\000\
    \109\000\113\000\012\000\255\255\255\255\011\000\032\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\012\000\255\255\
    \255\255\255\255\036\000\255\255\255\255\255\255\255\255\021\000\
    \255\255\255\255\255\255\012\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\012\000\027\000\027\000\027\000\027\000\027\000\
    \027\000\027\000\027\000\027\000\027\000\046\000\012\000\046\000\
    \255\255\255\255\046\000\046\000\046\000\046\000\046\000\046\000\
    \046\000\046\000\046\000\046\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\255\255\106\000\
    \000\000\040\000\108\000\255\255\010\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\106\000\
    \255\255\255\255\108\000\030\000\255\255\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\255\255\255\255\255\255\255\255\031\000\255\255\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\255\255\255\255\255\255\255\255\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\255\255\
    \255\255\255\255\036\000\255\255\255\255\041\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \255\255\255\255\255\255\255\255\043\000\255\255\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \044\000\044\000\044\000\044\000\044\000\044\000\044\000\044\000\
    \044\000\044\000\047\000\047\000\047\000\047\000\047\000\047\000\
    \047\000\047\000\047\000\047\000\044\000\044\000\044\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\047\000\255\255\
    \047\000\048\000\048\000\048\000\048\000\048\000\048\000\048\000\
    \048\000\048\000\048\000\255\255\255\255\106\000\255\255\255\255\
    \108\000\255\255\255\255\255\255\044\000\044\000\044\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\073\000\047\000\073\000\
    \047\000\255\255\073\000\073\000\073\000\073\000\073\000\073\000\
    \073\000\073\000\073\000\073\000\074\000\074\000\074\000\074\000\
    \074\000\074\000\074\000\074\000\074\000\074\000\085\000\085\000\
    \085\000\085\000\085\000\085\000\085\000\085\000\085\000\085\000\
    \074\000\074\000\074\000\075\000\255\255\075\000\075\000\075\000\
    \075\000\075\000\075\000\075\000\075\000\075\000\075\000\087\000\
    \087\000\087\000\087\000\087\000\087\000\087\000\087\000\087\000\
    \087\000\101\000\075\000\112\000\101\000\255\255\255\255\112\000\
    \074\000\074\000\074\000\076\000\076\000\076\000\076\000\076\000\
    \076\000\076\000\076\000\076\000\076\000\255\255\255\255\255\255\
    \255\255\101\000\112\000\255\255\076\000\076\000\076\000\076\000\
    \076\000\076\000\075\000\255\255\077\000\255\255\077\000\077\000\
    \077\000\077\000\077\000\077\000\077\000\077\000\077\000\077\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\112\000\077\000\076\000\076\000\076\000\076\000\
    \076\000\076\000\077\000\080\000\080\000\080\000\080\000\080\000\
    \080\000\080\000\080\000\080\000\080\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\101\000\080\000\080\000\080\000\080\000\
    \080\000\080\000\255\255\077\000\255\255\255\255\083\000\080\000\
    \083\000\255\255\077\000\083\000\083\000\083\000\083\000\083\000\
    \083\000\083\000\083\000\083\000\083\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\080\000\080\000\080\000\080\000\
    \080\000\080\000\255\255\255\255\110\000\111\000\255\255\080\000\
    \084\000\084\000\084\000\084\000\084\000\084\000\084\000\084\000\
    \084\000\084\000\086\000\086\000\086\000\086\000\086\000\086\000\
    \086\000\086\000\086\000\086\000\084\000\090\000\084\000\255\255\
    \255\255\255\255\090\000\255\255\110\000\111\000\086\000\255\255\
    \086\000\110\000\111\000\255\255\255\255\255\255\091\000\091\000\
    \091\000\091\000\091\000\091\000\091\000\091\000\091\000\091\000\
    \255\255\255\255\110\000\111\000\084\000\255\255\084\000\091\000\
    \091\000\091\000\091\000\091\000\091\000\255\255\086\000\114\000\
    \086\000\255\255\255\255\114\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\090\000\
    \255\255\255\255\255\255\255\255\255\255\090\000\114\000\255\255\
    \255\255\090\000\255\255\255\255\102\000\103\000\255\255\255\255\
    \255\255\090\000\255\255\255\255\255\255\090\000\255\255\090\000\
    \090\000\095\000\095\000\095\000\095\000\095\000\095\000\095\000\
    \095\000\095\000\095\000\255\255\102\000\103\000\114\000\101\000\
    \255\255\255\255\095\000\095\000\095\000\095\000\095\000\095\000\
    \096\000\096\000\096\000\096\000\096\000\096\000\096\000\096\000\
    \096\000\096\000\104\000\255\255\255\255\255\255\255\255\104\000\
    \255\255\096\000\096\000\096\000\096\000\096\000\096\000\097\000\
    \097\000\097\000\097\000\097\000\097\000\097\000\097\000\097\000\
    \097\000\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \097\000\097\000\097\000\097\000\097\000\097\000\102\000\103\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\104\000\255\255\255\255\255\255\
    \255\255\255\255\104\000\255\255\255\255\255\255\104\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\104\000\255\255\
    \255\255\255\255\104\000\255\255\104\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\110\000\111\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\102\000\103\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \104\000";
  Lexing.lex_base_code = 
   "";
  Lexing.lex_backtrk_code = 
   "";
  Lexing.lex_default_code = 
   "";
  Lexing.lex_trans_code = 
   "";
  Lexing.lex_check_code = 
   "";
  Lexing.lex_code = 
   "";
}

let rec token lexbuf =
    __ocaml_lex_token_rec lexbuf 0
and __ocaml_lex_token_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 195 "java/java_lexer.mll"
      ( token lexbuf )
# 520 "java/java_lexer.ml"

  | 1 ->
# 197 "java/java_lexer.mll"
      ( newline lexbuf; token lexbuf )
# 525 "java/java_lexer.ml"

  | 2 ->
# 199 "java/java_lexer.mll"
      ( let loc = lexeme_start_p lexbuf in
	Buffer.clear buf; ANNOT(loc,annot lexbuf) )
# 531 "java/java_lexer.ml"

  | 3 ->
let
# 201 "java/java_lexer.mll"
                        a
# 537 "java/java_lexer.ml"
= Lexing.sub_lexeme lexbuf (lexbuf.Lexing.lex_start_pos + 3) (lexbuf.Lexing.lex_curr_pos + -1) in
# 202 "java/java_lexer.mll"
      ( let loc = lexeme_start_p lexbuf in
	newline lexbuf;	ANNOT(loc,a) )
# 542 "java/java_lexer.ml"

  | 4 ->
# 205 "java/java_lexer.mll"
      ( comment lexbuf; token lexbuf )
# 547 "java/java_lexer.ml"

  | 5 ->
# 207 "java/java_lexer.mll"
      ( newline lexbuf; token lexbuf )
# 552 "java/java_lexer.ml"

  | 6 ->
let
# 208 "java/java_lexer.mll"
                                                            id
# 558 "java/java_lexer.ml"
= Lexing.sub_lexeme lexbuf (lexbuf.Lexing.lex_start_pos + 1) lexbuf.Lexing.lex_curr_pos in
# 209 "java/java_lexer.mll"
      ( special_id lexbuf id )
# 562 "java/java_lexer.ml"

  | 7 ->
let
# 210 "java/java_lexer.mll"
                                                      id
# 568 "java/java_lexer.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos lexbuf.Lexing.lex_curr_pos in
# 211 "java/java_lexer.mll"
      ( id_or_kw id )
# 572 "java/java_lexer.ml"

  | 8 ->
# 213 "java/java_lexer.mll"
      ( SEMICOLON )
# 577 "java/java_lexer.ml"

  | 9 ->
# 215 "java/java_lexer.mll"
      ( COMMA )
# 582 "java/java_lexer.ml"

  | 10 ->
# 217 "java/java_lexer.mll"
      ( DOT )
# 587 "java/java_lexer.ml"

  | 11 ->
# 219 "java/java_lexer.mll"
      ( PLUS )
# 592 "java/java_lexer.ml"

  | 12 ->
# 221 "java/java_lexer.mll"
      ( MINUS )
# 597 "java/java_lexer.ml"

  | 13 ->
# 223 "java/java_lexer.mll"
      ( PLUSPLUS )
# 602 "java/java_lexer.ml"

  | 14 ->
# 225 "java/java_lexer.mll"
      ( MINUSMINUS )
# 607 "java/java_lexer.ml"

  | 15 ->
# 227 "java/java_lexer.mll"
      ( STAR )
# 612 "java/java_lexer.ml"

  | 16 ->
# 229 "java/java_lexer.mll"
      ( SLASH )
# 617 "java/java_lexer.ml"

  | 17 ->
# 231 "java/java_lexer.mll"
      ( PERCENT )
# 622 "java/java_lexer.ml"

  | 18 ->
# 233 "java/java_lexer.mll"
      ( AMPERSAND )
# 627 "java/java_lexer.ml"

  | 19 ->
# 235 "java/java_lexer.mll"
      ( VERTICALBAR )
# 632 "java/java_lexer.ml"

  | 20 ->
# 237 "java/java_lexer.mll"
      ( AMPERSANDAMPERSAND )
# 637 "java/java_lexer.ml"

  | 21 ->
# 239 "java/java_lexer.mll"
      ( VERTICALBARVERTICALBAR )
# 642 "java/java_lexer.ml"

  | 22 ->
# 241 "java/java_lexer.mll"
      ( EQEQGT )
# 647 "java/java_lexer.ml"

  | 23 ->
# 243 "java/java_lexer.mll"
      ( LTEQEQGT )
# 652 "java/java_lexer.ml"

  | 24 ->
# 245 "java/java_lexer.mll"
      ( BANG )
# 657 "java/java_lexer.ml"

  | 25 ->
# 247 "java/java_lexer.mll"
      ( TILDA )
# 662 "java/java_lexer.ml"

  | 26 ->
# 249 "java/java_lexer.mll"
      ( CARET )
# 667 "java/java_lexer.ml"

  | 27 ->
# 251 "java/java_lexer.mll"
      ( QUESTIONMARK )
# 672 "java/java_lexer.ml"

  | 28 ->
# 253 "java/java_lexer.mll"
      ( COLON )
# 677 "java/java_lexer.ml"

  | 29 ->
# 255 "java/java_lexer.mll"
      ( SHIFT Blsl )
# 682 "java/java_lexer.ml"

  | 30 ->
# 257 "java/java_lexer.mll"
      ( SHIFT Blsr )
# 687 "java/java_lexer.ml"

  | 31 ->
# 259 "java/java_lexer.mll"
      ( SHIFT Basr )
# 692 "java/java_lexer.ml"

  | 32 ->
# 261 "java/java_lexer.mll"
      ( EQ )
# 697 "java/java_lexer.ml"

  | 33 ->
# 263 "java/java_lexer.mll"
      ( ASSIGNOP Bmul )
# 702 "java/java_lexer.ml"

  | 34 ->
# 265 "java/java_lexer.mll"
      ( ASSIGNOP Bdiv )
# 707 "java/java_lexer.ml"

  | 35 ->
# 267 "java/java_lexer.mll"
      ( ASSIGNOP Bmod )
# 712 "java/java_lexer.ml"

  | 36 ->
# 269 "java/java_lexer.mll"
      ( ASSIGNOP Badd )
# 717 "java/java_lexer.ml"

  | 37 ->
# 271 "java/java_lexer.mll"
      ( ASSIGNOP Bsub )
# 722 "java/java_lexer.ml"

  | 38 ->
# 273 "java/java_lexer.mll"
      ( ASSIGNOP Blsl )
# 727 "java/java_lexer.ml"

  | 39 ->
# 275 "java/java_lexer.mll"
      ( ASSIGNOP Blsr )
# 732 "java/java_lexer.ml"

  | 40 ->
# 277 "java/java_lexer.mll"
      ( ASSIGNOP Basr )
# 737 "java/java_lexer.ml"

  | 41 ->
# 279 "java/java_lexer.mll"
      ( ASSIGNOP Bbwand )
# 742 "java/java_lexer.ml"

  | 42 ->
# 281 "java/java_lexer.mll"
      ( ASSIGNOP Bbwxor )
# 747 "java/java_lexer.ml"

  | 43 ->
# 283 "java/java_lexer.mll"
      ( ASSIGNOP Bbwor )
# 752 "java/java_lexer.ml"

  | 44 ->
# 285 "java/java_lexer.mll"
      ( COMP Bgt )
# 757 "java/java_lexer.ml"

  | 45 ->
# 287 "java/java_lexer.mll"
      ( COMP Blt )
# 762 "java/java_lexer.ml"

  | 46 ->
# 289 "java/java_lexer.mll"
      ( COMP Ble )
# 767 "java/java_lexer.ml"

  | 47 ->
# 291 "java/java_lexer.mll"
      ( COMP Bge )
# 772 "java/java_lexer.ml"

  | 48 ->
# 293 "java/java_lexer.mll"
      ( EQOP Beq )
# 777 "java/java_lexer.ml"

  | 49 ->
# 295 "java/java_lexer.mll"
      ( EQOP Bne )
# 782 "java/java_lexer.ml"

  | 50 ->
let
# 299 "java/java_lexer.mll"
                                             n
# 788 "java/java_lexer.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos lexbuf.Lexing.lex_curr_pos in
# 300 "java/java_lexer.mll"
      ( INTCONSTANT n )
# 792 "java/java_lexer.ml"

  | 51 ->
let
# 304 "java/java_lexer.mll"
                               n
# 798 "java/java_lexer.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos lexbuf.Lexing.lex_curr_pos in
# 305 "java/java_lexer.mll"
      ( INTCONSTANT ("0o" ^ n) )
# 802 "java/java_lexer.ml"

  | 52 ->
let
# 309 "java/java_lexer.mll"
                                                      n
# 808 "java/java_lexer.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos lexbuf.Lexing.lex_curr_pos in
# 310 "java/java_lexer.mll"
    ( INTCONSTANT n )
# 812 "java/java_lexer.ml"

  | 53 ->
# 315 "java/java_lexer.mll"
      ( REALCONSTANT (lexeme lexbuf) )
# 817 "java/java_lexer.ml"

  | 54 ->
# 318 "java/java_lexer.mll"
      ( REALCONSTANT (lexeme lexbuf) )
# 822 "java/java_lexer.ml"

  | 55 ->
# 321 "java/java_lexer.mll"
      ( REALCONSTANT (lexeme lexbuf) )
# 827 "java/java_lexer.ml"

  | 56 ->
let
# 325 "java/java_lexer.mll"
                 s
# 833 "java/java_lexer.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos (lexbuf.Lexing.lex_start_pos + 3) in
# 326 "java/java_lexer.mll"
      ( CHARACTER s )
# 837 "java/java_lexer.ml"

  | 57 ->
let
# 328 "java/java_lexer.mll"
                                   s
# 843 "java/java_lexer.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos (lexbuf.Lexing.lex_start_pos + 4) in
# 329 "java/java_lexer.mll"
      ( CHARACTER s )
# 847 "java/java_lexer.ml"

  | 58 ->
let
# 332 "java/java_lexer.mll"
                                               s
# 853 "java/java_lexer.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos (lexbuf.Lexing.lex_start_pos + 8) in
# 333 "java/java_lexer.mll"
      ( CHARACTER s )
# 857 "java/java_lexer.ml"

  | 59 ->
# 336 "java/java_lexer.mll"
      ( LEFTPAR )
# 862 "java/java_lexer.ml"

  | 60 ->
# 338 "java/java_lexer.mll"
      ( RIGHTPAR )
# 867 "java/java_lexer.ml"

  | 61 ->
# 340 "java/java_lexer.mll"
      ( LEFTBRACE )
# 872 "java/java_lexer.ml"

  | 62 ->
# 342 "java/java_lexer.mll"
      ( RIGHTBRACE )
# 877 "java/java_lexer.ml"

  | 63 ->
# 344 "java/java_lexer.mll"
      ( LEFTBRACKET )
# 882 "java/java_lexer.ml"

  | 64 ->
# 346 "java/java_lexer.mll"
      ( RIGHTBRACKET )
# 887 "java/java_lexer.ml"

  | 65 ->
# 348 "java/java_lexer.mll"
      ( Buffer.clear buf; STRING(string lexbuf) )
# 892 "java/java_lexer.ml"

  | 66 ->
# 350 "java/java_lexer.mll"
      ( lex_error lexbuf ("unexpected char `" ^ lexeme lexbuf ^ "'") )
# 897 "java/java_lexer.ml"

  | 67 ->
# 352 "java/java_lexer.mll"
      ( EOF )
# 902 "java/java_lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_token_rec lexbuf __ocaml_lex_state

and string lexbuf =
    __ocaml_lex_string_rec lexbuf 101
and __ocaml_lex_string_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 356 "java/java_lexer.mll"
      ( Buffer.contents buf )
# 913 "java/java_lexer.ml"

  | 1 ->
# 358 "java/java_lexer.mll"
      ( Buffer.add_string buf (lexeme lexbuf);
	string lexbuf )
# 919 "java/java_lexer.ml"

  | 2 ->
# 361 "java/java_lexer.mll"
      ( lex_error lexbuf "unknown escape sequence in string" )
# 924 "java/java_lexer.ml"

  | 3 ->
# 363 "java/java_lexer.mll"
      ( (* no \n anymore in strings since java 1.4 *)
	lex_error lexbuf "string not terminated"; )
# 930 "java/java_lexer.ml"

  | 4 ->
# 366 "java/java_lexer.mll"
      ( Buffer.add_string buf (lexeme lexbuf); string lexbuf )
# 935 "java/java_lexer.ml"

  | 5 ->
# 368 "java/java_lexer.mll"
      ( lex_error lexbuf "string not terminated" )
# 940 "java/java_lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_string_rec lexbuf __ocaml_lex_state

and comment lexbuf =
    __ocaml_lex_comment_rec lexbuf 106
and __ocaml_lex_comment_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 372 "java/java_lexer.mll"
      ( () )
# 951 "java/java_lexer.ml"

  | 1 ->
# 374 "java/java_lexer.mll"
      ( newline lexbuf; comment lexbuf )
# 956 "java/java_lexer.ml"

  | 2 ->
# 376 "java/java_lexer.mll"
      ( comment lexbuf )
# 961 "java/java_lexer.ml"

  | 3 ->
# 378 "java/java_lexer.mll"
      ( comment lexbuf )
# 966 "java/java_lexer.ml"

  | 4 ->
# 380 "java/java_lexer.mll"
      ( lex_error lexbuf "comment not terminated" )
# 971 "java/java_lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_comment_rec lexbuf __ocaml_lex_state

and annot lexbuf =
    __ocaml_lex_annot_rec lexbuf 110
and __ocaml_lex_annot_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 384 "java/java_lexer.mll"
      ( Buffer.contents buf )
# 982 "java/java_lexer.ml"

  | 1 ->
# 386 "java/java_lexer.mll"
      ( newline lexbuf;  
	Buffer.add_string buf (lexeme lexbuf);
	annot lexbuf )
# 989 "java/java_lexer.ml"

  | 2 ->
let
# 389 "java/java_lexer.mll"
                    s
# 995 "java/java_lexer.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos (lexbuf.Lexing.lex_curr_pos + -1) in
# 390 "java/java_lexer.mll"
      ( newline lexbuf;  
	Buffer.add_string buf s;
	Buffer.add_char buf ' ';
	annot lexbuf )
# 1002 "java/java_lexer.ml"

  | 3 ->
# 395 "java/java_lexer.mll"
      ( Buffer.add_string buf (lexeme lexbuf);
	annot lexbuf )
# 1008 "java/java_lexer.ml"

  | 4 ->
# 398 "java/java_lexer.mll"
      ( Buffer.add_string buf (lexeme lexbuf);
	annot lexbuf )
# 1014 "java/java_lexer.ml"

  | 5 ->
# 401 "java/java_lexer.mll"
      ( lex_error lexbuf "annotation not terminated" )
# 1019 "java/java_lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_annot_rec lexbuf __ocaml_lex_state

;;

# 403 "java/java_lexer.mll"
 

let dotdot_mem = ref false
 
let next_token lexbuf =
  if !dotdot_mem then
    begin
      dotdot_mem := false;
      DOTDOT
    end
  else
    begin
      try
	token lexbuf
      with
	Dotdot(n) ->
	  dotdot_mem := true;
	  INTCONSTANT n
    end

  let parse f c =
    let lb = from_channel c in
    lb.lex_curr_p <- { lb.lex_curr_p with pos_fname = f };
    try
      Java_parser.compilation_unit next_token lb
    with Parsing.Parse_error ->
      Java_options.parsing_error (loc lb) ""


# 1055 "java/java_lexer.ml"
