# 2 "mix/mix_lexer.mll"
 
  open Lexing 
  open Mix_ast
  open Mix_parser

  let keywords = Hashtbl.create 97
  let () = 
    List.iter 
      (fun (x,y) -> Hashtbl.add keywords x y)
      [ (* loading *)
	"lda", INSTR (Ld A);  "ldx", INSTR (Ld X);
	"ld1", INSTR (Ld I1); "ld2", INSTR (Ld I2); "ld3", INSTR (Ld I3);
	"ld4", INSTR (Ld I4); "ld5", INSTR (Ld I5); "ld6", INSTR (Ld I6);
	"ldan", INSTR (Ldn A);  "ldxn", INSTR (Ldn X);
	"ld1n", INSTR (Ldn I1); "ld2n", INSTR (Ldn I2); "ld3n", INSTR (Ldn I3);
	"ld4n", INSTR (Ldn I4); "ld5n", INSTR (Ldn I5); "ld6n", INSTR (Ldn I6);
	(* storing *)
	"sta", INSTR (St A);  "stx", INSTR (St X);
	"st1", INSTR (St I1); "st2", INSTR (St I2); "st3", INSTR (St I3);
	"st4", INSTR (St I4); "st5", INSTR (St I5); "st6", INSTR (St I6);
	"stj", INSTR Stj;     "stz", INSTR Stz;
	(* arithmetic *)
	"add", INSTR Add; "sub", INSTR Sub; 
	"mul", INSTR Mul; "div", INSTR Div;
	(* address transfer *)
	"enta", INSTR (Ent A);  "entx", INSTR (Ent X);
	"ent1", INSTR (Ent I1); "ent2", INSTR (Ent I2); "ent3", INSTR (Ent I3);
	"ent4", INSTR (Ent I4); "ent5", INSTR (Ent I5); "ent6", INSTR (Ent I6);
	"enna", INSTR (Enn A);  "ennx", INSTR (Enn X);
	"enn1", INSTR (Enn I1); "enn2", INSTR (Enn I2); "enn3", INSTR (Enn I3);
	"enn4", INSTR (Enn I4); "enn5", INSTR (Enn I5); "enn6", INSTR (Enn I6);
	"inca", INSTR (Inc A);  "incx", INSTR (Inc X);
	"inc1", INSTR (Inc I1); "inc2", INSTR (Inc I2); "inc3", INSTR (Inc I3);
	"inc4", INSTR (Inc I4); "inc5", INSTR (Inc I5); "inc6", INSTR (Inc I6);
	"deca", INSTR (Dec A);  "decx", INSTR (Dec X);
	"dec1", INSTR (Dec I1); "dec2", INSTR (Dec I2); "dec3", INSTR (Dec I3);
	"dec4", INSTR (Dec I4); "dec5", INSTR (Dec I5); "dec6", INSTR (Dec I6);
        (* comparison *)
	"cmpa", INSTR (Cmp A);  "cmpx", INSTR (Cmp X);
	"cmp1", INSTR (Cmp I1); "cmp2", INSTR (Cmp I2); "cmp3", INSTR (Cmp I3);
	"cmp4", INSTR (Cmp I4); "cmp5", INSTR (Cmp I5); "cmp6", INSTR (Cmp I6);
	(* jumps *)
	"jmp", INSTR Jmp; "jsj", INSTR Jsj;
	"jge", INSTR Jge;
	"jan", INSTR (Jn A);  "jxn", INSTR (Jn X);
	"j1n", INSTR (Jn I1); "j2n", INSTR (Jn I2); "j3n", INSTR (Jn I3);
	"j4n", INSTR (Jn I4); "j5n", INSTR (Jn I5); "j6n", INSTR (Jn I6);
	"jaz", INSTR (Jz A);  "jxz", INSTR (Jz X);
	"j1z", INSTR (Jz I1); "j2z", INSTR (Jz I2); "j3z", INSTR (Jz I3);
	"j4z", INSTR (Jz I4); "j5z", INSTR (Jz I5); "j6z", INSTR (Jz I6);
	"jap", INSTR (Jp A);  "jxp", INSTR (Jp X);
	"j1p", INSTR (Jp I1); "j2p", INSTR (Jp I2); "j3p", INSTR (Jp I3);
	"j4p", INSTR (Jp I4); "j5p", INSTR (Jp I5); "j6p", INSTR (Jp I6);
	"jann", INSTR (Jnn A);  "jxnn", INSTR (Jnn X);
	"j1nn", INSTR (Jnn I1); "j2nn", INSTR (Jnn I2); "j3nn", INSTR (Jnn I3);
	"j4nn", INSTR (Jnn I4); "j5nn", INSTR (Jnn I5); "j6nn", INSTR (Jnn I6);
	"janz", INSTR (Jnz A);  "jxnz", INSTR (Jnz X);
	"j1nz", INSTR (Jnz I1); "j2nz", INSTR (Jnz I2); "j3nz", INSTR (Jnz I3);
	"j4nz", INSTR (Jnz I4); "j5nz", INSTR (Jnz I5); "j6nz", INSTR (Jnz I6);
	"janp", INSTR (Jnp A);  "jxnp", INSTR (Jnp X);
	"j1np", INSTR (Jnp I1); "j2np", INSTR (Jnp I2); "j3np", INSTR (Jnp I3);
	"j4np", INSTR (Jnp I4); "j5np", INSTR (Jnp I5); "j6np", INSTR (Jnp I6);
	(* other *)
	"nop", INSTR Nop;
	"hlt", INSTR Hlt;
	(* pseudo *)
	"equ", EQU;
	"orig", ORIG;
      ]

  let newline lexbuf =
    let pos = lexbuf.lex_curr_p in
    lexbuf.lex_curr_p <- 
      { pos with pos_lnum = pos.pos_lnum + 1; pos_bol = pos.pos_cnum }

  let string_buf = Buffer.create 1024

  let char_for_backslash = function
    | 'n' -> '\n'
    | 't' -> '\t'
    | c -> c

  exception Lexical_error of string

  (* local symbols *)

  let local_symbols = Array.create 10 (0,0)

  let make_local_symbol i n = Printf.sprintf "local_symbol_%d_%d" i n

  let line lexbuf = (lexeme_start_p lexbuf).pos_lnum

  let local_symbol lexbuf i =
    let (n,_) = local_symbols.(i) in
    local_symbols.(i) <- (n+1, line lexbuf);
    make_local_symbol i (n+1)

  let forward_local_symbol _ i =
    let (n,_) = local_symbols.(i) in
    make_local_symbol i (n+1)

  let backward_local_symbol lexbuf i =
    let (n,l) = local_symbols.(i) in
    make_local_symbol i (if line lexbuf > l then n else n-1)


# 109 "mix/mix_lexer.ml"
let __ocaml_lex_tables = {
  Lexing.lex_base = 
   "\000\000\234\255\235\255\236\255\237\255\238\255\239\255\240\255\
    \241\255\242\255\243\255\004\000\001\000\077\000\102\000\003\000\
    \255\255\252\255\177\000\249\255\250\255\001\000\251\255\012\000\
    \247\255\244\255\151\000\253\255\018\000\014\000\254\255\004\000\
    \001\000\011\000\012\000\013\000\014\000\015\000\017\000";
  Lexing.lex_backtrk = 
   "\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\021\000\010\000\007\000\002\000\001\000\
    \255\255\255\255\007\000\255\255\255\255\255\255\255\255\009\000\
    \255\255\255\255\255\255\255\255\004\000\004\000\255\255\255\255\
    \004\000\255\255\002\000\255\255\255\255\002\000\255\255";
  Lexing.lex_default = 
   "\001\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\255\255\255\255\255\255\255\255\255\255\
    \000\000\000\000\255\255\000\000\000\000\255\255\000\000\255\255\
    \000\000\000\000\022\000\000\000\255\255\255\255\000\000\022\000\
    \030\000\027\000\255\255\255\255\027\000\255\255\027\000";
  Lexing.lex_trans = 
   "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\015\000\016\000\000\000\015\000\015\000\027\000\000\000\
    \015\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \015\000\000\000\010\000\015\000\000\000\000\000\016\000\000\000\
    \007\000\006\000\003\000\005\000\009\000\004\000\025\000\011\000\
    \013\000\013\000\013\000\013\000\013\000\013\000\013\000\013\000\
    \013\000\013\000\008\000\022\000\030\000\016\000\000\000\000\000\
    \000\000\014\000\014\000\014\000\014\000\014\000\014\000\014\000\
    \014\000\014\000\014\000\014\000\014\000\014\000\014\000\014\000\
    \014\000\014\000\014\000\014\000\014\000\014\000\014\000\014\000\
    \014\000\014\000\014\000\000\000\000\000\000\000\000\000\014\000\
    \032\000\014\000\014\000\014\000\014\000\014\000\014\000\014\000\
    \014\000\014\000\014\000\014\000\014\000\014\000\014\000\014\000\
    \014\000\014\000\014\000\014\000\014\000\014\000\014\000\014\000\
    \014\000\014\000\014\000\012\000\023\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\024\000\
    \034\000\035\000\016\000\037\000\016\000\014\000\016\000\019\000\
    \000\000\000\000\000\000\020\000\000\000\021\000\014\000\014\000\
    \014\000\014\000\014\000\014\000\014\000\014\000\014\000\014\000\
    \017\000\027\000\000\000\000\000\000\000\000\000\000\000\014\000\
    \014\000\014\000\014\000\014\000\014\000\014\000\014\000\014\000\
    \014\000\014\000\014\000\014\000\014\000\014\000\014\000\014\000\
    \014\000\014\000\014\000\014\000\014\000\014\000\014\000\014\000\
    \014\000\029\000\000\000\000\000\000\000\014\000\028\000\014\000\
    \014\000\014\000\014\000\014\000\014\000\014\000\014\000\014\000\
    \014\000\014\000\014\000\014\000\014\000\014\000\014\000\014\000\
    \014\000\014\000\014\000\014\000\014\000\014\000\014\000\014\000\
    \014\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \002\000\255\255\000\000\000\000\017\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\030\000\000\000\000\000\030\000\000\000\
    \000\000\030\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\017\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000";
  Lexing.lex_check = 
   "\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\000\000\000\000\255\255\015\000\000\000\031\000\255\255\
    \015\000\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \000\000\255\255\000\000\015\000\255\255\255\255\031\000\255\255\
    \000\000\000\000\000\000\000\000\000\000\000\000\011\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\021\000\028\000\029\000\255\255\255\255\
    \255\255\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\255\255\255\255\255\255\255\255\000\000\
    \031\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\012\000\013\000\013\000\013\000\
    \013\000\013\000\013\000\013\000\013\000\013\000\013\000\023\000\
    \033\000\034\000\035\000\036\000\037\000\014\000\038\000\013\000\
    \255\255\255\255\255\255\013\000\255\255\013\000\014\000\014\000\
    \014\000\014\000\014\000\014\000\014\000\014\000\014\000\014\000\
    \014\000\026\000\255\255\255\255\255\255\255\255\255\255\014\000\
    \014\000\014\000\014\000\014\000\014\000\014\000\014\000\014\000\
    \014\000\014\000\014\000\014\000\014\000\014\000\014\000\014\000\
    \014\000\014\000\014\000\014\000\014\000\014\000\014\000\014\000\
    \014\000\026\000\255\255\255\255\255\255\014\000\026\000\014\000\
    \014\000\014\000\014\000\014\000\014\000\014\000\014\000\014\000\
    \014\000\014\000\014\000\014\000\014\000\014\000\014\000\014\000\
    \014\000\014\000\014\000\014\000\014\000\014\000\014\000\014\000\
    \014\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \000\000\032\000\255\255\255\255\031\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\033\000\255\255\255\255\036\000\255\255\
    \255\255\038\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\026\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255";
  Lexing.lex_base_code = 
   "";
  Lexing.lex_backtrk_code = 
   "";
  Lexing.lex_default_code = 
   "";
  Lexing.lex_trans_code = 
   "";
  Lexing.lex_check_code = 
   "";
  Lexing.lex_code = 
   "";
}

let rec token lexbuf =
    __ocaml_lex_token_rec lexbuf 0
and __ocaml_lex_token_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 119 "mix/mix_lexer.mll"
      ( newline lexbuf; token lexbuf )
# 262 "mix/mix_lexer.ml"

  | 1 ->
# 121 "mix/mix_lexer.mll"
      ( token lexbuf )
# 267 "mix/mix_lexer.ml"

  | 2 ->
let
# 122 "mix/mix_lexer.mll"
             id
# 273 "mix/mix_lexer.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos lexbuf.Lexing.lex_curr_pos in
# 123 "mix/mix_lexer.mll"
      ( let lid = String.lowercase id in
	try Hashtbl.find keywords lid with Not_found -> IDENT id )
# 278 "mix/mix_lexer.ml"

  | 3 ->
let
# 125 "mix/mix_lexer.mll"
             id
# 284 "mix/mix_lexer.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos (lexbuf.Lexing.lex_curr_pos + -1) in
# 126 "mix/mix_lexer.mll"
      ( LABEL id )
# 288 "mix/mix_lexer.ml"

  | 4 ->
let
# 127 "mix/mix_lexer.mll"
             d
# 294 "mix/mix_lexer.ml"
= Lexing.sub_lexeme_char lexbuf lexbuf.Lexing.lex_start_pos in
# 128 "mix/mix_lexer.mll"
      ( LABEL (local_symbol lexbuf (Char.code d - Char.code '0')) )
# 298 "mix/mix_lexer.ml"

  | 5 ->
let
# 129 "mix/mix_lexer.mll"
             d
# 304 "mix/mix_lexer.ml"
= Lexing.sub_lexeme_char lexbuf lexbuf.Lexing.lex_start_pos in
# 130 "mix/mix_lexer.mll"
      ( IDENT (forward_local_symbol lexbuf (Char.code d - Char.code '0')) )
# 308 "mix/mix_lexer.ml"

  | 6 ->
let
# 131 "mix/mix_lexer.mll"
             d
# 314 "mix/mix_lexer.ml"
= Lexing.sub_lexeme_char lexbuf lexbuf.Lexing.lex_start_pos in
# 132 "mix/mix_lexer.mll"
      ( IDENT (backward_local_symbol lexbuf (Char.code d - Char.code '0')) )
# 318 "mix/mix_lexer.ml"

  | 7 ->
let
# 133 "mix/mix_lexer.mll"
               n
# 324 "mix/mix_lexer.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos lexbuf.Lexing.lex_curr_pos in
# 134 "mix/mix_lexer.mll"
      ( INTEGER n )
# 328 "mix/mix_lexer.ml"

  | 8 ->
# 136 "mix/mix_lexer.mll"
      ( Buffer.clear string_buf; verbatim lexbuf )
# 333 "mix/mix_lexer.ml"

  | 9 ->
# 138 "mix/mix_lexer.mll"
      ( Buffer.clear string_buf; invariant lexbuf )
# 338 "mix/mix_lexer.ml"

  | 10 ->
# 140 "mix/mix_lexer.mll"
      ( Buffer.clear string_buf; assertion lexbuf )
# 343 "mix/mix_lexer.ml"

  | 11 ->
# 142 "mix/mix_lexer.mll"
      ( comment lexbuf; token lexbuf )
# 348 "mix/mix_lexer.ml"

  | 12 ->
# 143 "mix/mix_lexer.mll"
              ( Buffer.clear string_buf; string lexbuf )
# 353 "mix/mix_lexer.ml"

  | 13 ->
# 145 "mix/mix_lexer.mll"
      ( COMMA )
# 358 "mix/mix_lexer.ml"

  | 14 ->
# 147 "mix/mix_lexer.mll"
      ( COLON )
# 363 "mix/mix_lexer.ml"

  | 15 ->
# 149 "mix/mix_lexer.mll"
      ( LPAR )
# 368 "mix/mix_lexer.ml"

  | 16 ->
# 151 "mix/mix_lexer.mll"
      ( RPAR )
# 373 "mix/mix_lexer.ml"

  | 17 ->
# 153 "mix/mix_lexer.mll"
      ( PLUS )
# 378 "mix/mix_lexer.ml"

  | 18 ->
# 155 "mix/mix_lexer.mll"
      ( MINUS )
# 383 "mix/mix_lexer.ml"

  | 19 ->
# 157 "mix/mix_lexer.mll"
      ( STAR )
# 388 "mix/mix_lexer.ml"

  | 20 ->
# 159 "mix/mix_lexer.mll"
      ( EOF )
# 393 "mix/mix_lexer.ml"

  | 21 ->
let
# 160 "mix/mix_lexer.mll"
         c
# 399 "mix/mix_lexer.ml"
= Lexing.sub_lexeme_char lexbuf lexbuf.Lexing.lex_start_pos in
# 161 "mix/mix_lexer.mll"
      ( raise (Lexical_error ("illegal character: " ^ String.make 1 c)) )
# 403 "mix/mix_lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_token_rec lexbuf __ocaml_lex_state

and comment lexbuf =
    __ocaml_lex_comment_rec lexbuf 26
and __ocaml_lex_comment_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 165 "mix/mix_lexer.mll"
      ( () )
# 414 "mix/mix_lexer.ml"

  | 1 ->
# 167 "mix/mix_lexer.mll"
      ( comment lexbuf; comment lexbuf )
# 419 "mix/mix_lexer.ml"

  | 2 ->
# 169 "mix/mix_lexer.mll"
      ( newline lexbuf; comment lexbuf )
# 424 "mix/mix_lexer.ml"

  | 3 ->
# 171 "mix/mix_lexer.mll"
      ( raise (Lexical_error "unterminated comment") )
# 429 "mix/mix_lexer.ml"

  | 4 ->
# 173 "mix/mix_lexer.mll"
      ( comment lexbuf )
# 434 "mix/mix_lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_comment_rec lexbuf __ocaml_lex_state

and string lexbuf =
    __ocaml_lex_string_rec lexbuf 31
and __ocaml_lex_string_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 177 "mix/mix_lexer.mll"
      ( STRING (Buffer.contents string_buf) )
# 445 "mix/mix_lexer.ml"

  | 1 ->
let
# 178 "mix/mix_lexer.mll"
               c
# 451 "mix/mix_lexer.ml"
= Lexing.sub_lexeme_char lexbuf (lexbuf.Lexing.lex_start_pos + 1) in
# 179 "mix/mix_lexer.mll"
      ( Buffer.add_char string_buf (char_for_backslash c); string lexbuf )
# 455 "mix/mix_lexer.ml"

  | 2 ->
# 181 "mix/mix_lexer.mll"
      ( newline lexbuf; Buffer.add_char string_buf '\n'; string lexbuf )
# 460 "mix/mix_lexer.ml"

  | 3 ->
# 183 "mix/mix_lexer.mll"
      ( raise (Lexical_error "unterminated string") )
# 465 "mix/mix_lexer.ml"

  | 4 ->
let
# 184 "mix/mix_lexer.mll"
         c
# 471 "mix/mix_lexer.ml"
= Lexing.sub_lexeme_char lexbuf lexbuf.Lexing.lex_start_pos in
# 185 "mix/mix_lexer.mll"
      ( Buffer.add_char string_buf c; string lexbuf )
# 475 "mix/mix_lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_string_rec lexbuf __ocaml_lex_state

and verbatim lexbuf =
    __ocaml_lex_verbatim_rec lexbuf 33
and __ocaml_lex_verbatim_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 189 "mix/mix_lexer.mll"
      ( VERBATIM (Buffer.contents string_buf) )
# 486 "mix/mix_lexer.ml"

  | 1 ->
# 191 "mix/mix_lexer.mll"
      ( raise (Lexical_error "unterminated verbatim") )
# 491 "mix/mix_lexer.ml"

  | 2 ->
let
# 192 "mix/mix_lexer.mll"
         c
# 497 "mix/mix_lexer.ml"
= Lexing.sub_lexeme_char lexbuf lexbuf.Lexing.lex_start_pos in
# 193 "mix/mix_lexer.mll"
      ( Buffer.add_char string_buf c; verbatim lexbuf )
# 501 "mix/mix_lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_verbatim_rec lexbuf __ocaml_lex_state

and invariant lexbuf =
    __ocaml_lex_invariant_rec lexbuf 36
and __ocaml_lex_invariant_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 197 "mix/mix_lexer.mll"
      ( INVARIANT (Buffer.contents string_buf) )
# 512 "mix/mix_lexer.ml"

  | 1 ->
# 199 "mix/mix_lexer.mll"
      ( raise (Lexical_error "unterminated invariant") )
# 517 "mix/mix_lexer.ml"

  | 2 ->
let
# 200 "mix/mix_lexer.mll"
         c
# 523 "mix/mix_lexer.ml"
= Lexing.sub_lexeme_char lexbuf lexbuf.Lexing.lex_start_pos in
# 201 "mix/mix_lexer.mll"
      ( Buffer.add_char string_buf c; invariant lexbuf )
# 527 "mix/mix_lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_invariant_rec lexbuf __ocaml_lex_state

and assertion lexbuf =
    __ocaml_lex_assertion_rec lexbuf 38
and __ocaml_lex_assertion_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 205 "mix/mix_lexer.mll"
      ( ASSERT (Buffer.contents string_buf) )
# 538 "mix/mix_lexer.ml"

  | 1 ->
# 207 "mix/mix_lexer.mll"
      ( raise (Lexical_error "unterminated assert") )
# 543 "mix/mix_lexer.ml"

  | 2 ->
let
# 208 "mix/mix_lexer.mll"
         c
# 549 "mix/mix_lexer.ml"
= Lexing.sub_lexeme_char lexbuf lexbuf.Lexing.lex_start_pos in
# 209 "mix/mix_lexer.mll"
      ( Buffer.add_char string_buf c; assertion lexbuf )
# 553 "mix/mix_lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_assertion_rec lexbuf __ocaml_lex_state

;;

